from queue import Queue
from threading import Thread

from serial import *
import os


class Relay(object):
    def __init__(self, ports):
        self.serials = [Serial(port=p, baudrate="57600") for p in ports]
        self.threads = []
        self.alive = False

    def start(self):
        self.alive = True
        self.threads.append(Thread(target=self.transmit,args=(self.serials[0],self.serials[1])))
        self.threads.append(Thread(target=self.transmit, args=(self.serials[1], self.serials[0])))
        for thread in self.threads:
            thread.daemon = True
            thread.start()


    def stop(self):
        self.alive = False

    def join(self):
        for thread in self.threads:
            while thread.isAlive():
                thread.join(0.1)

    def transmit(self,source, target):
        source.write(bytes([115]))
        while self.alive:
            read = source.read()
            if ord(read) == 112:
                data = source.read()
                print("{0}-Print: {1}".format(str(source.port), str(chr(ord(data)))))
            else:
                if target:
                    target.write(read)
                    print("{0}-Message: {1}".format(str(source.port), str(read)))
                else:
                    print("{0}-Message: {1}".format(str(source.port), str(read)))


if __name__ == "__main__":
    relay = Relay(["COM14","COM17"])
    relay.start()
    relay.join()