/*
 * demo.c
 *
 *  Created on: Oct 4, 2015
 *      Author: petrovic
 */

#include "tatrabot.h"

/* test the internal and external LEDs */
static void testLEDs(void) {
  for (int i = 0; i < 7; i++) {
    setLED(i, 1);
    chThdSleepMilliseconds(400);
  }
  for (int i = 0; i < 7; i++) {
    setLED(i, 0);
    chThdSleepMilliseconds(400);
  }

  setLED(0, 1);
  chThdSleepMilliseconds(400);
  for (int i = 1; i < 7; i++) {
    setLED(i - 1, 0);
    setLED(i, 1);
    chThdSleepMilliseconds(400);
  }

  for (int i = 6; i >= 0; i--) {
    setLED(i, 0);
    setLED(i - 1, 1);
    chThdSleepMilliseconds(400);
  }
  setLED(0, 0);
}

/* test buttons K3, K4 */
static void testButtons(void) {
  chprintf(PC, "...please do K1-K2 restart when done...\r\n");
  while (true) {
    uint8_t but3 = senseButton(3);
    for (uint8_t i = 1; i < 4; i++)
      setLED(i, but3);
    uint8_t but4 = senseButton(4);
    for (uint8_t i = 4; i < 7; i++)
      setLED(i, but4);
  }
}

/* play that annoying infamous song */
static void testSound(void) {
  sound(262);
  chThdSleepMilliseconds(700);

  sound(294);
  chThdSleepMilliseconds(700);

  sound(330);
  chThdSleepMilliseconds(1400);

  sound(349);
  chThdSleepMilliseconds(1400);

  sound(349);
  chThdSleepMilliseconds(700);

  sound(349);
  chThdSleepMilliseconds(700);

  sound(349);
  chThdSleepMilliseconds(700);

  sound(330);
  chThdSleepMilliseconds(700);

  sound(294);
  chThdSleepMilliseconds(1400);

  sound(330);
  chThdSleepMilliseconds(1400);

  sound(330);
  chThdSleepMilliseconds(700);

  sound(330);
  chThdSleepMilliseconds(700);

  sound(330);
  chThdSleepMilliseconds(700);

  sound(294);
  chThdSleepMilliseconds(700);

  sound(262);
  chThdSleepMilliseconds(1400);

  sound(294);
  chThdSleepMilliseconds(1400);

  sound(294);
  chThdSleepMilliseconds(700);

  sound(294);
  chThdSleepMilliseconds(700);

  sound(294);
  chThdSleepMilliseconds(700);

  sound(330);
  chThdSleepMilliseconds(700);

  sound(294);
  chThdSleepMilliseconds(1400);

  sound(262);
  chThdSleepMilliseconds(1400);

  sound(262);
  chThdSleepMilliseconds(700);

  sound(262);
  chThdSleepMilliseconds(700);

  nosound();
}

/* light the LEDs depending on the state of the line sensors */
static void testLine(void) {
  while (!senseButton(3)) {
    uint8_t ln = senseLine();
    setLED(1, ln & 1);
    setLED(2, (ln >> 1) & 1);
    setLED(3, (ln >> 2) & 1);
    setLED(4, (ln >> 3) & 1);
    chprintf(PC, "%d %d %d %d\n\r", ln & 1, (ln >> 1) & 1, (ln >> 2) & 1,
             (ln >> 3) & 1);
  }
}

/* light the LEDs depending on the state of the line sensors */
static void testObstacle(void) {
  while (!senseButton(3)) {
    setLED(1, obstacleLeft());
    setLED(2, obstacleRight());
  }
}

/* test the SHARP distance sensor - also show the distance on the serial line 1 */
static void testSharp(void) {
  while (!senseButton(3)) {
    chprintf(PC, "d = %d\n\r", distance);
    sound(distance);
    chThdSleepMilliseconds(100);
  }
  nosound();
}

/* test the accelerometer - show the x,y,z values on serial line 1 */
static void testAcc(void) {
  while (!senseButton(3)) {
    senseAcc();
    chprintf(PC, "x = %5d, y = %5d, z = %5d\n\r", (int16_t)(acc_vals[0]),
             (int16_t)(acc_vals[1]), (int16_t)(acc_vals[2]));
    chThdSleepMilliseconds(100);
  }
}

/* test the gyroscope - show the x,y,z values on serial line 1 */
static void testGyro(void) {
  while (!senseButton(3)) {
    senseGyro();
    chprintf(PC, "x = %5d, y = %5d, z = %5d\n\r", (int16_t)(gyro_vals[0]),
             (int16_t)(gyro_vals[1]), (int16_t)(gyro_vals[2]));
    chThdSleepMilliseconds(100);
  }
}

/* test the magnetometer/compass - show the x,y,z values and dir on serial line 1 */
static void testCompass(void) {
  while (!senseButton(3)) {
    senseMagneto();
    chprintf(PC, "x = %5d, y = %5d, z = %5d, dir= %3.2f\n\r",
             (int16_t)(mag_vals[0]), (int16_t)(mag_vals[1]),
             (int16_t)(mag_vals[2]), compass());
    chThdSleepMilliseconds(100);
  }
}

/* test the rotation sensors - show the countA, countB on serial line 1 */
static void testICU(void) {
  static unsigned int last_countA, last_countB;

  while (!senseButton(3)) {
    if ((last_countA == countA) && (last_countB == countB))
      continue;
    chprintf(PC, "%5d\t%5d\r\n", countA, countB);
    last_countA = countA;
    last_countB = countB;
  }
}

/* test forward movement with simple regulation using rotation sensors */
static void regulatedFwd(unsigned int distance, int speed) {
  int speedL, speedR;
  speedL = speedR = speed;
  setMotor(3, speedL);
  countA = countB = 0;

  int iterationDelay = 50;
  while (countA < distance) {
    setMotor(1, speedL);
    setMotor(2, speedR);
    chThdSleepMilliseconds(iterationDelay);
    if (countA + countB > 16) {
      if (countA > countB + 1) {
        if (speedL < speed)
          speedL += 30;
        else
          speedR -= 30;
      }
      else if (countA < countB - 1) {
        if (speedR < speed)
          speedR += 30;
        else
          speedL -= 30;
      }
    }
  }
  motorsOff();
}

/* show some simple movements of the motors */
static void testMotors(void) {
  chprintf(PC, "\r\nleft fwd...\r\n");
  setMotor(1, 4000);
  chThdSleepMilliseconds(2000);
  chprintf(PC, "right fwd...\r\n");
  setMotor(1, 0);
  setMotor(2, 4000);
  chThdSleepMilliseconds(2000);

  chprintf(PC, "both fwd...\r\n");
  for (int i = 0; i < 10000; i += 3) {
    setMotor(3, i);
    chThdSleepMilliseconds(3);
  }
  chThdSleepMilliseconds(200);
  for (int i = 10000; i > 3500; i -= 3) {
    setMotor(3, i);
    chThdSleepMilliseconds(3);
  }
  for (int i = 0; i < 4; i++) {
    setMotor(3, 3500);
    chThdSleepMilliseconds(1000);
    setMotor(1, -3500);
    chThdSleepMilliseconds(2000);
  }
  setMotor(3, -3500);
  chThdSleepMilliseconds(2000);
  setMotor(1, 3500);
  chThdSleepMilliseconds(2000);
  for (int i = 3500; i > 0; i -= 3) {
    setMotor(3, i);
    chThdSleepMilliseconds(5);
  }
  motorsOff();
}
static void testRelayBlueTooth(void) {
  chprintf(
      PC,
      "please connect to the BT from your computer, and press K4 then...\r\n");
  while (!senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce

  chprintf(PC, "BT relay ON\r\n");
  while (senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce

  while (!senseButton(4)) {
    uint8_t chr;
    int nread = 1;
    int nread = sdAsynchronousRead(&SD3, &chr, 1);
    if (nread == 1)
      chSequentialStreamPut(&SD1, chr);
    int nread = sdAsynchronousRead(&SD1, &chr, 1);
    if (nread == 1)
      chSequentialStreamPut(&SD3, chr);
  }
  chThdSleepMilliseconds(100); //debounce
  while (senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce

  chprintf(
      PC,
      "now everything you type here will be echoed back to the BT terminal, press K3 when done...\r\n");
  while (senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce
  while (!senseButton(3)) {
    uint8_t chr = chSequentialStreamGet(PC);
    chSequentialStreamPut(BT, chr);
  }
}

static void testBlueTooth(void) {
  chprintf(
      PC,
      "please connect to the BT from your computer, and press K4 then...\r\n");
  while (!senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce

  chprintf(
      PC,
      "everything you type on the BT terminal will be echoed back here, press K4 when happy...\r\n");
  while (senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce

  while (!senseButton(4)) {
    uint8_t chr = chSequentialStreamGet(BT);
    chSequentialStreamPut(PC, chr);
  }
  chThdSleepMilliseconds(100); //debounce
  while (senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce

  chprintf(
      PC,
      "now everything you type here will be echoed back to the BT terminal, press K3 when done...\r\n");
  while (senseButton(4))
    ;
  chThdSleepMilliseconds(100); //debounce
  while (!senseButton(3)) {
    uint8_t chr = chSequentialStreamGet(PC);
    chSequentialStreamPut(BT, chr);
  }
}

static unsigned char demo_menu(BaseSequentialStream *term) {
  chprintf(term, " *** Tatrabot demo program ***\r\n");
  chprintf(term, "  a] test LEDs\r\n");
  chprintf(term, "  b] test buttons K3,K4\r\n");
  chprintf(term, "  c] test sound\r\n");
  chprintf(term, "  d] test distance sensor\r\n");
  chprintf(term, "  e] test obstacle sensors\r\n");
  chprintf(term, "  f] test line sensor\r\n");
  chprintf(term, "  g] test accelerometer\r\n");
  chprintf(term, "  h] test rotation sensors\r\n");
  chprintf(term, "  i] test motors\r\n");
  chprintf(term, "  j] test regulated movement\r\n");
  chprintf(term, "  k] test BlueTooth communication\r\n");
  chprintf(term, "  l] set term=BT\r\n");
  chprintf(term, "  m] set term=PC\r\n");
  chprintf(term, "  n] test gyroscope\r\n");
  chprintf(term, "  o] test compass\r\n");
  chprintf(term, "  p] test BT relay\r\n");
  chprintf(term, "\r\nyour selection> ");
  uint8_t choice = chSequentialStreamGet(term);
  chprintf(
      term,
      "%c\r\n\r\n...test in progress, wait for finish or terminate with button K3 or K1-K2 restart...",
      choice);
  return choice;
}

void testRegulatedFwd(void) {
  for (int i = 0; i < 5; i++) {
    chThdSleepMilliseconds(5000);
    regulatedFwd(64, 5000);
  }
}

/* a simple demo program that demonstrates the capabilities of Tatrabot */
void demo(void) {
  BaseSequentialStream *term = BT;
  while (true) {
    switch (demo_menu(term)) {
    case 'a':
      testLEDs();
      break;
    case 'b':
      testButtons();
      break;
    case 'c':
      testSound();
      break;
    case 'd':
      testSharp();
      break;
    case 'e':
      testObstacle();
      break;
    case 'f':
      testLine();
      break;
    case 'g':
      testAcc();
      break;
    case 'h':
      testICU();
      break;
    case 'i':
      testMotors();
      break;
    case 'j':
      testRegulatedFwd();
      break;
    case 'k':
      testBlueTooth();
      break;
    case 'l':
      term = BT;
      break;
    case 'm':
      term = PC;
      break;
    case 'n':
      testGyro();
      break;
    case 'o':
      testCompass();
      break;
    case 'p':
      testRelayBlueTooth();
      break;
    }
    chprintf(term, "test finished.\r\n\r\n");
  }
}
