/*
 * tatrabot.h
 *
 *  Created on: Oct 2, 2015
 *      Author: petrovic
 */

#ifndef TATRABOT_H_
#define TATRABOT_H_

#include "ch.h"
#include "hal.h"
#include "chprintf.h"

#define PC ((BaseSequentialStream *)&SD1)
#define BT ((BaseSequentialStream *)&SD3)

#define UNUSED(x) (void)(x)

/* turn the specified LED to state ON(1) or OFF(0)
 * LED0 = built-in blue LED
 * LED1 - LED6 = external LED module */
void setLED(uint8_t led, uint8_t state);

/* determine the state of the specified button, only buttons K3, and K4
 * can be determined, K1 is reset and K2 is BOOT0 */
uint8_t senseButton(int i);

/* turn off sound */
void nosound(void);

/* start playing sound of the specified frequency */
void sound(int freq);

/* sample the 4-channel binary line sensor, fills bits0..3 in the returned value */
uint8_t senseLine(void);

/* sample the current value of the left obstacle IR sensor: 0/1 */
uint8_t obstacleLeft(void);

/* sample the current value of the right obstacle IR sensor: 0/1 */
uint8_t obstacleRight(void);

/* last reading of the SHARP infrared sensor, it is measured automatically all the time */
extern volatile adcsample_t distance;

/* last reading of the accelerometer x,y,z values */
extern uint16_t acc_vals[3];

/* last reading of the gyroscope x,y,z values */
extern uint16_t gyro_vals[3];

/* last reading of the magnetometer x,y,z values */
extern uint16_t mag_vals[3];

/* measure the accelerometer values, result will be in acc_vals[] */
void senseAcc(void);

/* measure the gyroscope values, result will be in acc_vals[] */
void senseGyro(void);

/* measure the magnetometer values, result will be in acc_vals[] */
void senseMagneto(void);

/* convert the magnetometer readings to compass heading, must be called after
 * senseMagneto();
 */
double compass(void);

// current value of incremental counters A and B
extern volatile icucnt_t countA, countB;

/* control motor m=1 (left) or m=2 (right), 3 means to control both */
void setMotor(int m, int speed);

/* turn off the motors and disable the motor driver (save power) */
void motorsOff(void);

/* produce a greeting sound and flash the LEDs - called at the end of tatrabotInit() */
void startupGreeting(void);

/* initialize all the input/output devices, should be called at the start of each program */
void tatrabotInit(void);

#endif /* TATRABOT_H_ */
